@@ -9,15 +9,15 @@ use Furl;
 use Furl::ConnectionCache;
 
 use Scalar::Util ();
-use Errno qw(EAGAIN EINTR EWOULDBLOCK ECONNRESET);
+use Errno qw(EAGAIN ECONNRESET EINPROGRESS EINTR EWOULDBLOCK);
 use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK SEEK_SET SEEK_END);
 use Socket qw(
     PF_INET SOCK_STREAM
     IPPROTO_TCP
     TCP_NODELAY
-    inet_aton
     pack_sockaddr_in
 );
+use Time::HiRes qw(time);
 
 use constant WIN32 => $^O eq 'MSWin32';
 use HTTP::Parser::XS qw/HEADERS_NONE HEADERS_AS_ARRAYREF HEADERS_AS_HASHREF/;
@@ -54,6 +54,8 @@ sub new {
         no_proxy      => '',
         connection_pool     => Furl::ConnectionCache->new(),
         header_format => HEADERS_AS_ARRAYREF,
+        stop_if       => sub {},
+        inet_aton      => sub { Socket::inet_aton($_[0]) },
         %args
     }, $class;
 }
@@ -176,9 +178,7 @@ sub request {
     my $self = shift;
     my %args = @_;
 
-
-    my $timeout = $args{timeout};
-    $timeout = $self->{timeout} if not defined $timeout;
+    my $timeout_at = time + $self->{timeout};
 
     my ($scheme, $host, $port, $path_query);
     if (defined(my $url = $args{url})) {
@@ -225,45 +225,28 @@ sub request {
     my $sock         = $self->{connection_pool}->steal($host, $port);
     my $in_keepalive = defined $sock;
     if(!$in_keepalive) {
-        my ($_host, $_port);
+        my $err_reason;
         if ($proxy) {
-            (undef, $_host, $_port, undef)
+            my (undef, $proxy_host, $proxy_port, undef)
                 = $self->_parse_url($proxy);
-        }
-        else {
-            $_host = $host;
-            $_port = $port;
-        }
-
-        if ($scheme eq 'http') {
-            $sock = $self->connect($_host, $_port);
+            if ($scheme eq 'http') {
+                ($sock, $err_reason)
+                    = $self->connect($proxy_host, $proxy_port, $timeout_at);
+            } else {
+                ($sock, $err_reason) = $self->connect_ssl_over_proxy(
+                    $proxy_host, $proxy_port, $host, $port, $timeout_at);
+            }
         } else {
-            $sock = $proxy
-                ? $self->connect_ssl_over_proxy(
-                    $_host, $_port, $host, $port, $timeout)
-                : $self->connect_ssl($_host, $_port);
+            if ($scheme eq 'http') {
+                ($sock, $err_reason)
+                    = $self->connect($host, $port, $timeout_at);
+            } else {
+                ($sock, $err_reason)
+                    = $self->connect_ssl($host, $port, $timeout_at);
+            }
         }
-        setsockopt( $sock, IPPROTO_TCP, TCP_NODELAY, 1 )
-          or Carp::croak("Failed to setsockopt(TCP_NODELAY): $!");
-        if (WIN32) {
-            my $tmp = 1;
-            ioctl( $sock, 0x8004667E, \$tmp )
-              or Carp::croak("Cannot set flags for the socket: $!");
-        } else {
-            my $flags = fcntl( $sock, F_GETFL, 0 )
-              or Carp::croak("Cannot get flags for the socket: $!");
-            $flags = fcntl( $sock, F_SETFL, $flags | O_NONBLOCK )
-              or Carp::croak("Cannot set flags for the socket: $!");
-        }
-
-        {
-            # no buffering
-            my $orig = select();
-            select($sock); $|=1;
-            select($orig);
-        }
-
-        binmode $sock;
+        return $self->_r500($err_reason)
+            unless $sock;
     }
 
     # write request
@@ -320,8 +303,9 @@ sub request {
             $p .= "$headers[$i]: $val\015\012";
         }
         $p .= "\015\012";
-        $self->write_all($sock, $p, $timeout)
-            or return $self->_r500("Failed to send HTTP request: $!");
+        $self->write_all($sock, $p, $timeout_at)
+            or return $self->_r500(
+                "Failed to send HTTP request: " . _strerror_or_timeout());
         if (defined $content) {
             if ($content_is_fh) {
                 my $ret;
@@ -333,13 +317,17 @@ sub request {
                     } elsif ($ret == 0) { # EOF
                         last SENDFILE;
                     }
-                    $self->write_all($sock, $buf, $timeout)
-                        or return $self->_r500("Failed to send content: $!");
+                    $self->write_all($sock, $buf, $timeout_at)
+                        or return $self->_r500(
+                            "Failed to send content: " . _strerror_or_timeout()
+                        );
                 }
             } else { # simple string
                 if (length($content) > 0) {
-                    $self->write_all($sock, $content, $timeout)
-                        or return $self->_r500("Failed to send content: $!");
+                    $self->write_all($sock, $content, $timeout_at)
+                        or return $self->_r500(
+                            "Failed to send content: " . _strerror_or_timeout()
+                        );
                 }
             }
         }
@@ -360,7 +348,7 @@ sub request {
     $special_headers->{'transfer-encoding'} = '';
   LOOP: while (1) {
         my $n = $self->read_timeout($sock,
-            \$buf, $self->{bufsize}, length($buf), $timeout );
+            \$buf, $self->{bufsize}, length($buf), $timeout_at);
         if(!$n) { # error or eof
             if($in_keepalive && (defined($n) || $!==ECONNRESET)) {
                 # the server closes the connection (maybe because of keep-alive timeout)
@@ -368,7 +356,7 @@ sub request {
             }
             return $self->_r500(
                 !defined($n)
-                    ? "Cannot read response header: $!"
+                    ? "Cannot read response header: " . _strerror_or_timeout()
                     : "Unexpected EOF while reading response header"
             );
         }
@@ -417,17 +405,17 @@ sub request {
         my @err;
         if ( $chunked ) {
             @err = $self->_read_body_chunked($sock,
-                \$res_content, $rest_header, $timeout);
+                \$res_content, $rest_header, $timeout_at);
         } else {
             $res_content .= $rest_header;
             if (ref $res_content || !defined($content_length)) {
                 @err = $self->_read_body_normal($sock,
                     \$res_content, length($rest_header),
-                    $content_length, $timeout);
+                    $content_length, $timeout_at);
             } else {
                 @err = $self->_read_body_normal_to_string_buffer($sock,
                     \$res_content, length($rest_header),
-                    $content_length, $timeout);
+                    $content_length, $timeout_at);
             }
         }
         if(@err) {
@@ -481,16 +469,34 @@ sub request {
 
 # connects to $host:$port and returns $socket
 sub connect :method {
-    my($self, $host, $port) = @_;
+    my($self, $host, $port, $timeout_at) = @_;
     my $sock;
-    my $iaddr = inet_aton($host)
-        or Carp::croak("Cannot resolve host name: $host, $!");
+
+    my $timeout = $timeout_at - time;
+    return (undef, "Failed to resolve host name: timeout")
+        if $timeout <= 0;
+    my $iaddr = $self->{inet_aton}->($host, $timeout)
+        or return (undef, "Cannot resolve host name: $host, $!");
+
     my $sock_addr = pack_sockaddr_in($port, $iaddr);
 
+ RETRY:
     socket($sock, PF_INET, SOCK_STREAM, 0)
         or Carp::croak("Cannot create socket: $!");
-    connect($sock, $sock_addr)
-        or Carp::croak("Cannot connect to ${host}:${port}: $!");
+    _set_sockopts($sock);
+    if (connect($sock, $sock_addr)) {
+        # connected
+    } elsif ($! == EINPROGRESS) {
+        $self->do_select(1, $sock, $timeout_at)
+            or return (undef, "Cannot connect to ${host}:${port}: timeout");
+        # connected
+    } else {
+        if ($! == EINTR && ! $self->{stop_if}->()) {
+            close $sock;
+            goto RETRY;
+        }
+        return (undef, "Cannot connect to ${host}:${port}: $!");
+    }
     return $sock;
 }
 
@@ -498,39 +504,53 @@ sub connect :method {
 # You can override this methond in your child class, if you want to use Crypt::SSLeay or some other library.
 # @return file handle like object
 sub connect_ssl {
-    my ($self, $host, $port) = @_;
+    my ($self, $host, $port, $timeout_at) = @_;
     _requires('IO/Socket/SSL.pm', 'SSL');
 
-    return IO::Socket::SSL->new( PeerHost => $host, PeerPort => $port )
-      or Carp::croak("Cannot create SSL connection: $!");
+    my $timeout = $timeout_at - time;
+    return (undef, "Cannot create SSL connection: timeout")
+        if $timeout <= 0;
+    my $sock = IO::Socket::SSL->new(
+        PeerHost => $host,
+        PeerPort => $port,
+        Timeout  => $timeout,
+    ) or return (undef, "Cannot create SSL connection: $!");
+    _set_sockopts($sock);
+    $sock;
 }
 
 sub connect_ssl_over_proxy {
-    my ($self, $proxy_host, $proxy_port, $host, $port, $timeout) = @_;
+    my ($self, $proxy_host, $proxy_port, $host, $port, $timeout_at) = @_;
     _requires('IO/Socket/SSL.pm', 'SSL');
 
-    my $sock = $self->connect($proxy_host, $proxy_port);
+    my $sock = $self->connect($proxy_host, $proxy_port, $timeout_at);
 
     my $p = "CONNECT $host:$port HTTP/1.0\015\012Server: $host\015\012\015\012";
-    $self->write_all($sock, $p, $timeout)
-        or return $self->_r500("Failed to send HTTP request to proxy: $!");
+    $self->write_all($sock, $p, $timeout_at)
+        or return $self->_r500(
+            "Failed to send HTTP request to proxy: " . _strerror_or_timeout());
     my $buf = '';
     my $read = $self->read_timeout($sock,
-        \$buf, $self->{bufsize}, length($buf), $timeout);
+        \$buf, $self->{bufsize}, length($buf), $timeout_at);
     if (not defined $read) {
-        Carp::croak("Cannot read proxy response: $!");
+        return (undef, "Cannot read proxy response: " . _strerror_or_timeout());
     } elsif ( $read == 0 ) {    # eof
-        Carp::croak("Unexpected EOF while reading proxy response");
+        return (undef, "Unexpected EOF while reading proxy response");
     } elsif ( $buf !~ /^HTTP\/1.[01] 200 Connection established\015\012/ ) {
-        Carp::croak("Invalid HTTP Response via proxy");
+        return (undef, "Invalid HTTP Response via proxy");
     }
 
+    my $timeout = $timeout_at - time;
+    return (undef, "Cannot start SSL connection: timeout")
+        if $timeout_at <= 0;
     IO::Socket::SSL->start_SSL( $sock, Timeout => $timeout )
-      or Carp::croak("Cannot start SSL connection: $!");
+      or return (
+          undef, "Cannot start SSL connection: " . _strerror_or_timeout());
+    _set_sockopts($sock); # just in case (20101118 kazuho)
 }
 
 sub _read_body_chunked {
-    my ($self, $sock, $res_content, $rest_header, $timeout) = @_;
+    my ($self, $sock, $res_content, $rest_header, $timeout_at) = @_;
 
     my $buf = $rest_header;
   READ_LOOP: while (1) {
@@ -560,9 +580,10 @@ sub _read_body_chunked {
             # +2 means trailing CRLF
           READ_CHUNK: while ( $next_len+2 > length($buf) ) {
                 my $n = $self->read_timeout( $sock,
-                    \$buf, $self->{bufsize}, length($buf), $timeout );
+                    \$buf, $self->{bufsize}, length($buf), $timeout_at );
                 if ( not defined $n ) {
-                    return $self->_r500("Cannot read chunk: $!");
+                    return $self->_r500(
+                        "Cannot read chunk: " . _strerror_or_timeout());
                 }
             }
             $$res_content .= substr($buf, 0, $next_len);
@@ -573,29 +594,30 @@ sub _read_body_chunked {
         }
 
         my $n = $self->read_timeout( $sock,
-            \$buf, $self->{bufsize}, length($buf), $timeout );
+            \$buf, $self->{bufsize}, length($buf), $timeout_at );
         if (!$n) {
             return $self->_r500(
                 !defined($n)
-                    ? "Cannot read chunk: $!"
+                    ? "Cannot read chunk: " . _strerror_or_timeout()
                     : "Unexpected EOF while reading packets"
             );
         }
     }
     # read last CRLF
     return $self->_read_body_normal(
-        $sock, \$buf, length($buf), 2, $timeout);
+        $sock, \$buf, length($buf), 2, $timeout_at);
 }
 
 sub _read_body_normal {
-    my ($self, $sock, $res_content, $nread, $res_content_length, $timeout) = @_;
+    my ($self, $sock, $res_content, $nread, $res_content_length, $timeout_at)
+        = @_;
     while (!defined($res_content_length) || $res_content_length != $nread) {
         my $n = $self->read_timeout( $sock,
-            \my $buf, $self->{bufsize}, 0, $timeout );
+            \my $buf, $self->{bufsize}, 0, $timeout_at );
         if (!$n) {
             return $self->_r500(
                 !defined($n)
-                    ? "Cannot read content body: $!"
+                    ? "Cannot read content body: " . _strerror_or_timeout()
                     : "Unexpected EOF while reading content body"
             );
         }
@@ -608,14 +630,15 @@ sub _read_body_normal {
 # This function loads all content at once if it's possible. Since $res_content is just a plain scalar.
 # Buffering is not needed.
 sub _read_body_normal_to_string_buffer {
-    my ($self, $sock, $res_content, $nread, $res_content_length, $timeout) = @_;
+    my ($self, $sock, $res_content, $nread, $res_content_length, $timeout_at)
+        = @_;
     while ($res_content_length != $nread) {
         my $n = $self->read_timeout( $sock,
-            $res_content, $res_content_length, $nread, $timeout );
+            $res_content, $res_content_length, $nread, $timeout_at );
         if (!$n) {
             return $self->_r500(
                 !defined($n)
-                    ? "Cannot read content body: $!"
+                    ? "Cannot read content body: " . _strerror_or_timeout()
                     : "Unexpected EOF while reading content body"
             );
         }
@@ -624,12 +647,16 @@ sub _read_body_normal_to_string_buffer {
     return;
 }
 
-# I/O with tmeout (stolen from Starlet/kazuho++)
-
+# returns true if the socket is ready to read, false if timeout has occurred ($! will be cleared upon timeout)
 sub do_select {
-    my($self, $is_write, $sock, $timeout) = @_;
+    my($self, $is_write, $sock, $timeout_at) = @_;
     # wait for data
     while (1) {
+        my $timeout = $timeout_at - time;
+        if ($timeout <= 0) {
+            $! = 0;
+            return 0;
+        }
         my($rfd, $wfd);
         my $efd = '';
         vec($efd, fileno($sock), 1) = 1;
@@ -638,56 +665,61 @@ sub do_select {
         } else {
             $rfd = $efd;
         }
-        my $start_at = time;
         my $nfound   = select($rfd, $wfd, $efd, $timeout);
-        return 1 if $nfound;
-        $timeout    -= (time - $start_at);
-        return 0 if $timeout <= 0;
+        return 1 if $nfound > 0;
+        return 0 if $nfound == -1 && $! == EINTR && $self->{stop_if}->();
     }
     die 'not reached';
 }
 
 # returns (positive) number of bytes read, or undef if the socket is to be closed
 sub read_timeout {
-    my ($self, $sock, $buf, $len, $off, $timeout) = @_;
+    my ($self, $sock, $buf, $len, $off, $timeout_at) = @_;
     my $ret;
-    $self->do_select(0, $sock, $timeout) or return undef;
+    $self->do_select(0, $sock, $timeout_at) or return undef;
     while(1) {
         # try to do the IO
         defined($ret = sysread($sock, $$buf, $len, $off))
             and return $ret;
-
-        unless ($! == EINTR || $! == EAGAIN || $! == EWOULDBLOCK) {
+        if ($! == EAGAIN || $! == EWOULDBLOCK) {
+            # pass thru
+        } elsif ($! == EINTR) {
+            return undef if $self->{stop_if}->();
+            # otherwise pass thru
+        } else {
             return undef;
         }
         # on EINTER/EAGAIN/EWOULDBLOCK
-        $self->do_select(0, $sock, $timeout) or return undef;
+        $self->do_select(0, $sock, $timeout_at) or return undef;
     }
 }
 
 # returns (positive) number of bytes written, or undef if the socket is to be closed
 sub write_timeout {
-    my ($self, $sock, $buf, $len, $off, $timeout) = @_;
+    my ($self, $sock, $buf, $len, $off, $timeout_at) = @_;
     my $ret;
     while(1) {
         # try to do the IO
         defined($ret = syswrite($sock, $buf, $len, $off))
             and return $ret;
-
-        unless ($! == EINTR || $! == EAGAIN || $! == EWOULDBLOCK) {
+        if ($! == EAGAIN || $! == EWOULDBLOCK) {
+            # pass thru
+        } elsif ($! == EINTR) {
+            return undef if $self->{stop_if}->();
+            # otherwise pass thru
+        } else {
             return undef;
         }
-        # on EINTER/EAGAIN/EWOULDBLOCK
-        $self->do_select(1, $sock, $timeout) or return undef;
+        $self->do_select(1, $sock, $timeout_at) or return undef;
     }
 }
 
 # writes all data in buf and returns number of bytes written or undef if failed
 sub write_all {
-    my ($self, $sock, $buf, $timeout) = @_;
+    my ($self, $sock, $buf, $timeout_at) = @_;
     my $off = 0;
     while (my $len = length($buf) - $off) {
-        my $ret = $self->write_timeout($sock, $buf, $len, $off, $timeout)
+        my $ret = $self->write_timeout($sock, $buf, $len, $off, $timeout_at)
             or return undef;
         $off += $ret;
     }
@@ -702,6 +734,36 @@ sub _r500 {
         ['Content-Length' => length($message)], $message);
 }
 
+sub _strerror_or_timeout {
+    $! != 0 ? "$!" : 'timeout';
+}
+
+sub _set_sockopts {
+    my $sock = shift;
+
+    setsockopt( $sock, IPPROTO_TCP, TCP_NODELAY, 1 )
+        or Carp::croak("Failed to setsockopt(TCP_NODELAY): $!");
+    if (WIN32) {
+        my $tmp = 1;
+        ioctl( $sock, 0x8004667E, \$tmp )
+            or Carp::croak("Cannot set flags for the socket: $!");
+    } else {
+        my $flags = fcntl( $sock, F_GETFL, 0 )
+            or Carp::croak("Cannot get flags for the socket: $!");
+        $flags = fcntl( $sock, F_SETFL, $flags | O_NONBLOCK )
+            or Carp::croak("Cannot set flags for the socket: $!");
+    }
+
+    {
+        # no buffering
+        my $orig = select();
+        select($sock); $|=1;
+        select($orig);
+    }
+
+    binmode $sock;
+}
+
 # You can override this method if you want to use more powerful matcher.
 sub match_no_proxy {
     my ( $self, $no_proxy, $host ) = @_;
@@ -811,6 +873,8 @@ I<%args> might be:
 
 =item timeout :Int = 10
 
+Seconds until the call to $furl->request returns a timeout error (as an internally generated 500 error). The timeout might not be accurate since some underlying modules / built-ins function may block longer than the specified timeout. See the FAQ for how to support timeout during name resolution.
+
 =item max_redirects :Int = 7
 
 =item proxy :Str
@@ -835,6 +899,14 @@ This is the connection pool object for keep-alive requests. By default, it is a
 
 You may not customize this variable otherwise to use L<Coro>. This attribute requires a duck type object. It has two methods, C<< $obj->steal($host, $port >> and C<< $obj->push($host, $port, $sock) >>.
 
+=item stop_if
+
+A callback function that is called by Furl after when a blocking function call returns EINTR. Furl will abort the HTTP request and return immediately if the callback returns true. Otherwise the operation is continued (the default behaviour).
+
+=item inet_aton
+
+A callback function to customize name resolution. Takes two arguments: ($hostname, $timeout_in_seconds). If omitted, Furl calls L<Socket::inet_aton>.
+
 =back
 
 =head2 Instance Methods
@@ -997,6 +1069,17 @@ such cases.
 Anyway, the HEAD method is not so useful nowadays. The GET method and
 C<If-Modified-Sinse> are more suitable to cache HTTP contents.
 
+=item Why does Furl take longer than specified until it returns a timeout error?
+
+Although Furl itself supports timeout, some underlying modules / functions do not. And the most noticeable one is L<Socket::inet_aton>, the function used for name resolution (a function that converts hostnames to IP addresses). If you need accurate and short timeout for name resolution, the use of L<Net::DNS::Lite> is recommended. The following code snippet describes how to use the module in conjunction with Furl.
+
+    use Net::DNS::Lite qw();
+
+    my $furl = Furl->new(
+        timeout   => $my_timeout_in_seconds,
+        inet_aton => sub { Net::DNS::Lite::inet_aton(@_) },
+    );
+
 =back
 
 =head1 TODO
